@@ -1,5 +1,15 @@
-# noinspection PyProtectedMember
+"""
+The implementation below uses code adapted from the `asdict` helper function
+from the library Dataclasses (https://github.com/ericvsmith/dataclasses).
+
+This library is available under the Apache 2.0 license, which can be
+obtained from http://www.apache.org/licenses/LICENSE-2.0.
+
+
+See the end of this file for the original Apache license from this library.
+"""
 from collections import defaultdict
+# noinspection PyProtectedMember
 from dataclasses import _is_dataclass_instance
 from datetime import datetime, time, date
 from decimal import Decimal
@@ -10,10 +20,14 @@ from uuid import UUID
 from .abstractions import AbstractDumper
 from .bases import BaseDumpHook
 from .class_helper import (
-    dataclass_fields, get_class, dataclass_field_to_json_field)
+    create_new_class,
+    dataclass_fields, dataclass_field_to_json_field,
+    dataclass_to_dumper, set_class_dumper,
+    setup_dump_config_for_cls_if_needed,
+)
 from .constants import _DUMP_HOOKS
 from .log import LOG
-from .type_def import NoneType, DD
+from .type_def import NoneType, DD, LS
 from .utils.string_conv import to_camel_case
 
 
@@ -71,6 +85,13 @@ class DumpMixin(AbstractDumper, BaseDumpHook):
 
         return typ(_asdict_inner(v, dict_factory, hooks) for v in o)
 
+    @staticmethod
+    def dump_with_set(
+            o: Union[LS], _typ: Type[LS],
+            dict_factory, hooks):
+
+        return list(_asdict_inner(v, dict_factory, hooks) for v in o)
+
     @staticmethod
     def dump_with_named_tuple(
             o: NamedTupleMeta, typ: Type[NamedTupleMeta],
@@ -129,6 +150,8 @@ def setup_default_dumper(cls=DumpMixin):
     # Complex types
     cls.register_dump_hook(Enum, cls.dump_with_enum)
     cls.register_dump_hook(UUID, cls.dump_with_uuid)
+    cls.register_dump_hook(set, cls.dump_with_set)
+    cls.register_dump_hook(frozenset, cls.dump_with_set)
     cls.register_dump_hook(list, cls.dump_with_list_or_tuple)
     cls.register_dump_hook(tuple, cls.dump_with_list_or_tuple)
     cls.register_dump_hook(NamedTupleMeta, cls.dump_with_named_tuple)
@@ -141,14 +164,30 @@ def setup_default_dumper(cls=DumpMixin):
     cls.register_dump_hook(date, cls.dump_with_date)
 
 
-def get_dumper(class_or_instance=None) -> Type[DumpMixin]:
+def get_dumper(class_or_instance=None, create=False) -> Type[DumpMixin]:
     """
-    Get the dumper for the class, or use the default one if none exists.
-    """
-    if hasattr(class_or_instance, _DUMP_HOOKS):
-        return get_class(class_or_instance)
+    Get the dumper for the class, using the following logic:
 
-    return DumpMixin
+        * Return the class if it's already a sub-class of :class:`DumpMixin`
+        * If `create` is enabled, a new sub-class of :class:`DumpMixin` for
+          the class will be generated and cached on the initial run.
+        * Otherwise, we will return the base dumper, :class:`DumpMixin`, which
+          can potentially be shared by more than one dataclass.
+
+    """
+    try:
+        return dataclass_to_dumper(class_or_instance)
+
+    except KeyError:
+
+        if hasattr(class_or_instance, _DUMP_HOOKS):
+            return set_class_dumper(class_or_instance, class_or_instance)
+
+        elif create:
+            cls_dumper = create_new_class(class_or_instance, (DumpMixin, ))
+            return set_class_dumper(class_or_instance, cls_dumper)
+
+        return set_class_dumper(class_or_instance, DumpMixin)
 
 
 def asdict(obj, *, dict_factory=dict) -> Dict[str, Any]:
@@ -173,15 +212,26 @@ def asdict(obj, *, dict_factory=dict) -> Dict[str, Any]:
     if not _is_dataclass_instance(obj):
         raise TypeError("asdict() should be called on dataclass instances")
 
+    # -- The following lines are added, and are not present in original implementation. --
     cls_dumper = get_dumper(obj)
     hooks = cls_dumper.__DUMP_HOOKS__
 
+    # Pass the class reference itself, since we know we have an instance of
+    # the class.
+    setup_dump_config_for_cls_if_needed(type(obj))
+
     # Call the optional hook that runs before we process the dataclass
     cls_dumper.__pre_as_dict__(obj)
+    # -- END --
 
     return _asdict_inner(obj, dict_factory, hooks)
 
 
+# NOTE: This method has been modified to accept a `hook` argument, and the
+# return type has been annotated as `Any`. The logic inside this method has
+# also been heavily modified from the original implementation in
+# `dataclasses`. However, I will call out specific lines where it is taken
+# directly from the original version.
 def _asdict_inner(obj, dict_factory, hooks) -> Any:
 
     typ = type(obj)
@@ -191,6 +241,7 @@ def _asdict_inner(obj, dict_factory, hooks) -> Any:
     if dump_hook is not None:
         return dump_hook(*hook_args)
 
+    # -- This check is the same as in the original version --
     if _is_dataclass_instance(obj):
 
         dataclass_to_json_field = dataclass_field_to_json_field(obj)
@@ -198,6 +249,7 @@ def _asdict_inner(obj, dict_factory, hooks) -> Any:
         result = []
 
         for f in dataclass_fields(obj):
+            # -- This line is *mostly* the same as in the original version --
             value = _asdict_inner(getattr(obj, f.name), dict_factory, hooks)
 
             json_field = dataclass_to_json_field.get(f.name)
@@ -207,12 +259,14 @@ def _asdict_inner(obj, dict_factory, hooks) -> Any:
                 json_field = cls_dumper.transform_dataclass_field(f.name)
                 dataclass_to_json_field[f.name] = json_field
 
+            # -- This line is *mostly* the same as in the original version --
             result.append((json_field, value))
-
+        # -- This line is the same as in the original version --
         return dict_factory(result)
 
     else:
 
+        # -- The following `if` condition and comments are the same as in the original version --
         if isinstance(obj, tuple) and hasattr(obj, '_fields'):
             # obj is a namedtuple.  Recurse into it, but the returned
             # object is another namedtuple of the same type.  This is
@@ -230,3 +284,18 @@ def _asdict_inner(obj, dict_factory, hooks) -> Any:
         LOG.warning('Using default dumper, object=%r, type=%r', obj, typ)
 
         return DumpMixin.default_dump_with(*hook_args)
+
+
+# Copyright 2017-2018 Eric V. Smith
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
